"""
Statistic Code Repository Tests.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from models import (StatisticCode)
from repositories import StatisticCodeRepository


def create_maker() -> sessionmaker:
    """
    Creates the session maker.
    :return: sessionmaker
    """

    engine = create_engine('sqlite://')
    StatisticCode.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_statistic_code_exists():
    """
    Tests Statistic Code Exists.
    """

    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.statistic_code_exists(code)
    assert_that(result).is_true()


def test_statistic_code_not_exists():
    """
    Tests Statistics Code Not Exists.
    """
    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)

    result = repo.statistic_code_exists(code2)
    assert_that(result).is_false()


def test_get_statistic_code():
    """
    Test retrieving the statistic code by the id.
    """
    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_statistic_code(id=1)
    assert_that(result).is_equal_to(code)


def test_get_statistic_code_not_exists():
    """
    Test retrieving statistic code that doesn't exist
    """

    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_statistic_code(id=3)
    assert_that(result).is_none()


def test_get_statistic_code_no_args():
    """
    Tests retrieving a statistic code without args.
    """

    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_statistic_code()
    assert_that(result).is_none()


def test_get_statistic_code_by_code():
    """
    Tests retrieving a statistic code by code.
    """
    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_statistic_code(code='TST2')
    assert_that(result).is_equal_to(code2)


def test_get_statistic_codes():
    """
    Tests retrieving the statistic codes.
    """
    maker = create_maker()
    code = StatisticCode(id=1, code='TST', description='Test Code')
    code2 = StatisticCode(id=2, code='TST2', description='Test Code2')
    repo = StatisticCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_statistic_codes()
    assert_that(result).contains_only(code, code2)



"""
Repository Tests for Position Codes.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from models import Position
from repositories import PositionCodeRepository


def create_maker() -> sessionmaker:
    """
    Creates the Session Maker.
    """
    engine = create_engine('sqlite://')
    Position.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_position_code_exists():
    """
    Tests if the Position Code Exits.
    """

    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    repo.save(position)

    result = repo.position_code_exists(position)

    assert_that(result).is_true()


def test_position_code_does_not_exist():
    """
    Tests if a position code does not exist.
    """
    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    position2 = Position(id=2, code='WR', description='Wide Receiver')
    repo.save(position)

    result = repo.position_code_exists(position2)
    assert_that(result).is_false()


def test_get_position_code_by_id():
    """
    Tests retrieving a Position Code by ID Value
    """
    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    repo.save(position)

    result = repo.get_position_code(id=1)
    assert_that(result).is_equal_to(position)


def test_get_position_code_by_code():
    """
    Tests retrieving a position code by the code.
    """
    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    repo.save(position)

    result = repo.get_position_code(code='QB')
    assert_that(result).is_equal_to(position)


def test_get_position_code_not_exist():
    """
    Tests retrieving a code that does not exist.
    """
    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    repo.save(position)

    result = repo.get_position_code(code='WR')
    assert_that(result).is_none()


def test_get_position_code_no_args():
    """
    Tests retrieving the Position code without args
    """
    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    repo.save(position)

    result = repo.get_position_code()
    assert_that(result).is_none()


def test_get_position_codes():
    """
    Tests retrieving all Position Codes.
    """
    maker = create_maker()
    repo = PositionCodeRepository(maker)

    position = Position(id=1, code='QB', description='Quarterback')
    repo.save(position)

    result = repo.get_position_codes()
    assert_that(result).contains(position)



"""
Tests for the Schedule Repository.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from models import Schedule
from repositories import ScheduleRepository


def create_maker() -> sessionmaker:
    engine = create_engine('sqlite://')
    Schedule.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_schedule_exists():
    """
    Tests if a schedule exits.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)

    result = repo.schedule_exists(schedule)
    assert_that(result).is_true()


def test_schedule_does_not_exist():
    """
    Tests if a schedule does not exit.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    schedule2 = Schedule(id=2, team_id=2, opponent_id=1, year_value=2020, week_number=3,
                         game_id=665567,
                         url='www.google.com', type_id=1, is_home=False)

    repo = ScheduleRepository(maker)
    repo.save(schedule)

    result = repo.schedule_exists(schedule2)
    assert_that(result).is_false()


def test_get_schedule():
    """
    Tests retrieving a Schedule.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)

    repo = ScheduleRepository(maker)
    repo.save(schedule)

    result = repo.get_schedule(1)
    assert_that(result).is_equal_to(schedule)


def test_get_schedule_not_exists():
    """
    Tests retrieving a schedule that does not exist.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)

    result = repo.get_schedule(2)
    assert_that(result).is_none()


def test_get_schedules():
    """
    Tests retrieving all schedules.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)

    result = repo.get_schedules()
    assert_that(result).is_not_empty().contains(schedule)


def test_get_schedules_by_team():
    """
    Tests retrieving a Teams schedules.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)
    result = repo.get_schedules(team_id=1)
    assert_that(result).is_not_empty().contains(schedule)


def test_get_schedules_by_year_and_week():
    """
    Tests retrieving Schedules by Week and Year
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    schedule2 = Schedule(id=2, team_id=2, opponent_id=1, year_value=2020, week_number=4,
                         game_id=665567,
                         url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)
    repo.save(schedule2)

    result = repo.get_schedules(year=2020, week=3)
    assert_that(result).is_not_empty().contains_only(schedule)


def test_get_schedules_by_team_year_week():
    """
    Tests retrieving a Teams Schedule by week and year.
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    schedule2 = Schedule(id=2, team_id=2, opponent_id=1, year_value=2020, week_number=4,
                         game_id=665567,
                         url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)
    repo.save(schedule2)

    result = repo.get_schedules(team_id=1, year=2020, week=3)
    assert_that(result).is_not_empty().contains_only(schedule)


def test_get_schedules_by_team_year():
    """
    Tests retrieving Schedule by Team and Year
    """

    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    schedule2 = Schedule(id=2, team_id=2, opponent_id=1, year_value=2020, week_number=4,
                         game_id=665567,
                         url='www.google.com', type_id=1, is_home=True)
    schedule3 = Schedule(id=3, team_id=2, opponent_id=1, year_value=2021, week_number=4,
                         game_id=665568,
                         url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)
    repo.save(schedule2)
    repo.save(schedule3)

    result = repo.get_schedules(team_id=2, year=2020)
    assert_that(result).contains_only(schedule2)


def test_get_schedules_by_year():
    """
    Tests retrieving the Schedules by only the year.
    :return:
    """
    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    schedule2 = Schedule(id=2, team_id=2, opponent_id=1, year_value=2020, week_number=4,
                         game_id=665567,
                         url='www.google.com', type_id=1, is_home=True)
    schedule3 = Schedule(id=3, team_id=2, opponent_id=1, year_value=2021, week_number=4,
                         game_id=665568,
                         url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)
    repo.save(schedule2)
    repo.save(schedule3)

    result = repo.get_schedules(year=2020)

    assert_that(result).contains_only(schedule, schedule2)


def test_get_schedule_by_week():
    """
    Tests retrieving schedule by only week.
    """

    maker = create_maker()
    schedule = Schedule(id=1, team_id=1, opponent_id=2, year_value=2020, week_number=3,
                        game_id=665566,
                        url='www.google.com', type_id=1, is_home=True)
    schedule2 = Schedule(id=2, team_id=2, opponent_id=1, year_value=2020, week_number=4,
                         game_id=665567,
                         url='www.google.com', type_id=1, is_home=True)
    schedule3 = Schedule(id=3, team_id=2, opponent_id=1, year_value=2021, week_number=4,
                         game_id=665568,
                         url='www.google.com', type_id=1, is_home=True)
    repo = ScheduleRepository(maker)
    repo.save(schedule)
    repo.save(schedule2)
    repo.save(schedule3)

    result = repo.get_schedules(week=4)
    assert_that(result).contains_only(schedule2, schedule3)



"""
Tests for the Type Code Repository.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from models import TypeCode
from repositories import TypeCodeRepository


def create_maker() -> sessionmaker:
    """
    Creates a Session Maker.
    """
    engine = create_engine('sqlite://')
    TypeCode.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_type_code_exists():
    """
    Tests the Type Code Exists.
    """
    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    repo = TypeCodeRepository(maker)
    repo.save(code)

    result = repo.type_code_exists(code)
    assert_that(result).is_true()


def test_type_code_not_exists():
    """
    Tests the Type Code not exists.
    """
    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    code2 = TypeCode(id=2, code='TST2', description='Test Code 2')
    repo = TypeCodeRepository(maker)
    repo.save(code)

    result = repo.type_code_exists(code2)
    assert_that(result).is_false()


def test_get_type_code():
    """
    Tests retrieving Type Code by ID
    """
    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    code2 = TypeCode(id=2, code='TST2', description='Test Code 2')
    repo = TypeCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_type_code(id=1)
    assert_that(result).is_equal_to(code)


def test_get_type_code_by_code():
    """
    Tests retrieving Type code by Code.
    """
    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    code2 = TypeCode(id=2, code='TST2', description='Test Code 2')
    repo = TypeCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_type_code(code='TST')
    assert_that(result).is_equal_to(code)


def test_get_type_code_not_exist():
    """
    Tests retrieving a Type Code that does not exist.
    """
    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    code2 = TypeCode(id=2, code='TST2', description='Test Code 2')
    repo = TypeCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_type_code(id=3)
    assert_that(result).is_none()


def test_get_type_code_no_args():
    """
    Tests retrieving a Type Code without Args.
    """

    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    code2 = TypeCode(id=2, code='TST2', description='Test Code 2')
    repo = TypeCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_type_code()
    assert_that(result).is_none()


def test_get_type_codes():
    """
    Tests retrieving all type codes.
    """
    maker = create_maker()
    code = TypeCode(id=1, code='TST', description='Test Code')
    code2 = TypeCode(id=2, code='TST2', description='Test Code 2')
    repo = TypeCodeRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_type_codes()
    assert_that(result).is_not_empty().contains_only(code, code2)



"""
Team Staff Repository Tests.
"""
from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from models import TeamStaff
from repositories import TeamStaffRepository


def create_maker() -> sessionmaker:
    """
    Creates the Session Maker.
    :return: Session Maker
    """

    engine = create_engine('sqlite://')
    TeamStaff.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_team_staff_exists():
    """
    Tests if a Team Staff Exists.
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)

    result = repo.team_staff_exists(staff)
    assert_that(result).is_true()


def test_team_staff_does_not_exist():
    """
    Tests that a Team Staff does not exist.
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    staff2 = TeamStaff(id=2, player_id=2, team_id=2, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)

    result = repo.team_staff_exists(staff2)
    assert_that(result).is_false()


def test_get_team_staff_entry():
    """
    Tests retrieving a Team Staff from by ID
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    staff2 = TeamStaff(id=2, player_id=2, team_id=2, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)
    repo.save(staff2)

    result = repo.get_team_staff_entry(1)
    assert_that(result).is_equal_to(staff)


def test_get_team_staff_entries():
    """
    Tests retrieving all Team Staff Entries
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    staff2 = TeamStaff(id=2, player_id=2, team_id=2, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)
    repo.save(staff2)

    result = repo.get_team_staff_entries()
    assert_that(result).contains_only(staff, staff2)


def test_get_team_staff_team_id():
    """
    Tests retrieving the Team Staff entries by Team Id
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    staff2 = TeamStaff(id=2, player_id=2, team_id=3, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)
    repo.save(staff2)

    result = repo.get_team_staff_entries(team_id=3)
    assert_that(result).contains_only(staff2)


def test_get_team_staff_by_player_team():
    """
    Tests retrieving the Team Staff by Player and Team ID
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    staff2 = TeamStaff(id=2, player_id=2, team_id=2, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)
    repo.save(staff2)

    result = repo.get_team_staff_entries(player_id=1, team_id=2)
    assert_that(result).contains_only(staff)


def test_get_team_staff_by_player():
    """
    Tests retrieving the Team Staff by Player Id
    """
    maker = create_maker()
    staff = TeamStaff(id=1, player_id=1, team_id=2, year_value=2020)
    staff2 = TeamStaff(id=2, player_id=2, team_id=2, year_value=2020)
    repo = TeamStaffRepository(maker)
    repo.save(staff)
    repo.save(staff2)

    result = repo.get_team_staff_entries(player_id=1)
    assert_that(result).contains_only(staff)



"""
Tests for the Team League Repository.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from models import TeamLeague
from repositories import TeamLeagueRepository


def create_maker() -> sessionmaker:
    """
    Creates the Session Maker.
    :return: Session Maker
    """

    engine = create_engine('sqlite://')
    TeamLeague.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_team_league_exists():
    """
    Tests the Team League exists.
    """
    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)

    result = repo.team_league_exists(team_league)
    assert_that(result).is_true()


def test_team_league_not_exist():
    """
    Tests the Team League does not exist.
    """

    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    team_league2 = TeamLeague(id=2, team_id=1, league_id=2, year_value=2022)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)

    result = repo.team_league_exists(team_league2)
    assert_that(result).is_false()


def test_get_team_leagues_team():
    """
    Test returning the Team Leagues by Team.
    """
    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    team_league2 = TeamLeague(id=2, team_id=2, league_id=2, year_value=2022)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)
    repo.save(team_league2)

    result = repo.get_team_leagues(team_id=1)
    assert_that(result).contains_only(team_league)


def test_get_team_league_league():
    """
    Tests returning the Team Leagues by Leage.
    """
    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    team_league2 = TeamLeague(id=2, team_id=1, league_id=2, year_value=2022)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)
    repo.save(team_league2)

    result = repo.get_team_leagues(league_id=2)
    assert_that(result).contains_only(team_league, team_league2)


def test_get_team_leagues():
    """
    Tests returning all Team Leagues.
    """
    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    team_league2 = TeamLeague(id=2, team_id=1, league_id=2, year_value=2022)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)
    repo.save(team_league2)

    result = repo.get_team_leagues()
    assert_that(result).contains_only(team_league, team_league2)


def test_get_team_league():
    """
    Tests returning the Team League by ID.
    """
    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    team_league2 = TeamLeague(id=2, team_id=1, league_id=2, year_value=2022)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)
    repo.save(team_league2)

    result = repo.get_team_league(1)
    assert_that(result).is_equal_to(team_league)


def test_get_team_league_not_exist():
    """
    Tests returning the Team League that does not exist.
    """
    maker = create_maker()
    team_league = TeamLeague(id=1, team_id=1, league_id=2, year_value=2021)
    team_league2 = TeamLeague(id=2, team_id=1, league_id=2, year_value=2022)
    repo = TeamLeagueRepository(maker)
    repo.save(team_league)
    repo.save(team_league2)

    result = repo.get_team_league(3)
    assert_that(result).is_none()
